//------------------------------------------------------------------------------
#include "stm32f4xx.h"
#include <stdio.h>
#include <string.h>
//------------------------------------------------------------------------------
#define DECODE_MODE_REG     0x09
#define INTESITY_REG        0x0A
#define SCAN_LIMIT_REG      0x0B
#define SHUTDOWN_REG        0x0C
#define DISPLAY_TEST_REG    0x0F
//------------------------------------------------------------------------------
#define DISABLE_DECODE      0x00
#define DECODE_ALL          0xFF
#define BRIGHTNESS          0x0F
#define SCAN_LOW_NIBBLE     0x03
#define SCAN_ALL_DIGITS     0x07   
#define SHUTDOWN_MODE       0x00
#define NORMAL_OPERATION    0x01
#define DISABLE_TEST_MODE   0x00
#define ENABLE_TEST_MODE    0x01
#define DECIMAL_POINT       0x80
//------------------------------------------------------------------------------
void MAX7219_init(void);
void MAX7219_write(char reg,signed char data);
void MAX7219_displayNumber(signed int number);
char getDigit(char pos, char number);
//------------------------------------------------------------------------------
void SPI_init(void);
void SPI_write(signed char data);
char SPI_read(void);
//------------------------------------------------------------------------------
void HSI_init(void);
void delay_ms(int n);
void delay_us (int time); 
//------------------------------------------------------------------------------
int main()
{	
	HSI_init();
	MAX7219_init();
		
	while(1)
	{
		MAX7219_displayNumber(287);
		delay_ms(2000);
	}
}
//------------------------------------------------------------------------------
void HSI_init(void)  //Set Clock to 16Mhz
{
	// Set the HSION bit and wait for HSIRDY
	RCC->CR |= 1ul << 0;
	while(!(RCC->CR & 2ul)){};

  // System clock switch to HSI		
  RCC->CFGR &= ~(3ul << 0);   // HSI oscillator selected as system clock
	RCC->CFGR &= ~(7ul << 27);  // MCO2 Prescaler. NO DIVISION selected 
	RCC->CFGR &= ~(3ul << 30);  // System clock (SYSCLK) selected
}
//------------------------------------------------------------------------------
void SPI_init(void)
{	
	//GPIOA Clock Enable
	RCC->AHB1ENR |= RCC_AHB1ENR_GPIOAEN;  
	
	//Configure PA0 (Serial Data Out)
	GPIOA->MODER &= ~0x00000003;
	GPIOA->MODER |= 0x00000001;       	//SDOUT is Output
	
	//Configure PA1 (Chip Select)
	GPIOA->MODER &= ~0x00000000C;
	GPIOA->MODER |= 0x00000004;					//CS is Output
	
	//Configure PA2 (Serial Clock)
	GPIOA->MODER &= ~0x000000030;
	GPIOA->MODER |= 0x00000010;					//SCLK is Output
	
	//Configure PA3 (Serial Data In)
	GPIOA->MODER &= ~0x0000000C0;				//SDIN is input				
}
//--------------------------TX: transmit data-----------------------------------
void SPI_write(signed char data)
{
  for(char Tx = 0 ; Tx < 8 ; Tx++ )
  {
		GPIOA->BSRR = GPIO_BSRR_BR2;			//SCK = 0
		GPIOA->BSRR = ((data << Tx) & 0x80) ? GPIO_BSRR_BS0 : GPIO_BSRR_BR0 ; //SDOUT (MSB first).
		GPIOA->BSRR = GPIO_BSRR_BS2;			//SCK = 1
  }
}
//--------------------------RX: recieve data------------------------------------
char SPI_read(void)
{
  char data = 0;

  for(char Rx = 0 ; Rx < 8 ; Rx++ )
  {
		GPIOA->BSRR = GPIO_BSRR_BR2;			//SCK = 0
		data += ((GPIOA->IDR & GPIO_IDR_ID3) << (7-Rx)); 	//SDIN (MSB first).
		GPIOA->BSRR = GPIO_BSRR_BS2;			//SCK = 1
  }

  return data;
}
//------------------------------------------------------------------------------
void MAX7219_init(void)
{
  SPI_init();
  
  MAX7219_write(DECODE_MODE_REG, DECODE_ALL);    
  MAX7219_write(INTESITY_REG, BRIGHTNESS);     
  MAX7219_write(SCAN_LIMIT_REG, SCAN_LOW_NIBBLE);     
  MAX7219_write(SHUTDOWN_REG, NORMAL_OPERATION);
}
//------------------------------------------------------------------------------
void MAX7219_write(char reg, signed char data)
{	
	GPIOA->BSRR = GPIO_BSRR_BR1;			//CS = 0
	
	SPI_write(reg); 
  SPI_write(data); 
	
	GPIOA->BSRR = GPIO_BSRR_BS1;			//CS = 1
}
//------------------------------------------------------------------------------
void MAX7219_displayNumber(signed int number)
{   
  char strNum[15]; 
  sprintf(strNum,"%4i",number);
  
  for(char pos = 0; pos < strlen(strNum); pos++)
  {   
    char i = strNum[pos];  
    MAX7219_write( pos+1, getDigit(pos,i) );
  }
}
//------------------------------------------------------------------------------
char getDigit(char pos, char number)
{
    char digit = 0;
    
    if(number >= '0' && number <= '9') 
    {
        digit = (pos == 2) ? (number + DECIMAL_POINT) : number ;
    }
    else
        if(number == '-')
        {
            digit = (pos == 2) ? (0 + DECIMAL_POINT) : 0x0A;  //0x0A - minus symbol see MAX7219 datasheet
        }
        else
            if(number == ' ')
            {
                digit = (pos == 2) ? (0 + DECIMAL_POINT) : 0x0F; //0x0F - blank. used for leading zero suppression
            }
    
    return digit;
}
//------------------------------------------------------------------------------
void delay_ms(int n) 
{
  int i;
  for(; n > 0; n--)
    for(i = 0; i < 3195; i++);
}
//------------------------------------------------------------------------------
void delay_us(int time)
{
	for(int i = 0; i < time; i++)
		for(int j = 0; j < 7; j++);
}
//------------------------------------------------------------------------------
